// Color areas are defined here. All the areas are made with DOM type 'path'. They are drawn to indicate the locations of the cell.

import { selectAll, line, scaleSequential, min, max, interpolateBlues, interpolateReds, arc, curveCardinal } from 'd3';

// Fetch required variables from 'index.js' (with 'Javascript object destructuring' syntax)
export const cellColorArea = (selection, props) => {
  const {
    height,
    width,
    margin,
    locationData,
    onClick,
    selectedLocation
  } = props;
  
  // define inner width and height
  const innerWidth = width - margin.right - margin.left;
  const innerHeight = height - margin.top - margin.bottom;

  // Color Scale Definitions
    // calculate minimum net_DEG values for the blue & red coloring purpose
  let minAbsDataValue = Math.abs(min(locationData, d=>+d.net_DEG));
  let maxAbsDataValue = Math.abs(max(locationData, d=>+d.net_DEG));

  const blueScale = scaleSequential(interpolateBlues)  // max absolute value between 'max net_DEG' and 'min net_DEG'
    .domain([0, -max([minAbsDataValue, maxAbsDataValue])]);
  const redScale = scaleSequential(interpolateReds)
    .domain([0, max([minAbsDataValue, maxAbsDataValue])]);

// Draw color areas from here
  // line generator function used for drawing the paths
  const lineFunction = line()
    .x(d => d.x)
    .y(d => d.y)
    .curve(curveCardinal);

// parent 'cellDiagram' group -> child each '.component' group
  const cellAreaG = selection.selectAll('.cellAreaDiagram').data([null]);
  const cellAreaGEnter = cellAreaG
    .enter()
      .append('g')
        .attr('class', 'cellAreaDiagram')
        .attr('transform', `translate(${(margin.left-52.8)}, ${margin.top-40})`);

//extracellular space
const extracellularData = [
  {"x": innerWidth*0, "y": innerHeight*0.3}, {"x": innerWidth*0.05, "y": innerHeight*0.05}, {"x": innerWidth*0.2, "y": innerHeight*0},
  {"x": innerWidth*0.8, "y": innerHeight*0}, {"x": innerWidth*0.95, "y": innerHeight*0.05}, {"x": innerWidth*1, "y": innerHeight*0.3},
  {"x": innerWidth*1, "y": innerHeight*0.7}, {"x": innerWidth*0.95, "y": innerHeight*0.95}, {"x": innerWidth*0.8, "y": innerHeight*1},
  {"x": innerWidth*0.2, "y": innerHeight*1}, {"x": innerWidth*0.05, "y": innerHeight*0.95}, {"x": innerWidth*0, "y": innerHeight*0.7},
  {"x": innerWidth*0, "y": innerHeight*0.3},
  {"x": 100, "y": 70}, {"x": 69, "y": 140}, {"x": 62, "y": 200}, {"x": 76, "y": 260}, {"x": 120, "y": 320}, {"x": 170, "y": 350}, {"x": 270, "y": 365}, {"x": 600, "y": 365}, {"x": 695, "y": 334}, {"x": 745, "y": 280}, {"x": 769, "y": 210}, {"x": 763, "y": 140}, {"x": 721, "y": 70}, {"x": 650, "y": 25}, {"x": 550, "y": 17}, {"x": 240, "y": 18}, {"x": 150, "y": 40}, {"x": 82, "y": 91},
  {"x": innerWidth*0, "y": innerHeight*0.3}
];
  const extracellularSpace = cellAreaGEnter
    .append('path')
      .attr('id', 'extracellular space')
      .attr('class', 'component')
      .attr('d', lineFunction(extracellularData));

//plasma membrane
const membraneData = [
  {"x": 131, "y": 85}, {"x": 155, "y": 70}, {"x": 175, "y": 58}, {"x": 215, "y": 46}, {"x": 315, "y": 46}, {"x": 600, "y": 46}, {"x": 662, "y": 57}, {"x": 683, "y": 70}, {"x": 700, "y": 86}, {"x": 734, "y": 140}, {"x": 743, "y": 200}, {"x": 725, "y": 260}, {"x": 694, "y": 300}, {"x": 646, "y": 330}, {"x": 600, "y": 338}, {"x": 550, "y": 339}, {"x": 220, "y": 337}, {"x": 136, "y": 300}, {"x": 101, "y": 250}, {"x": 89, "y": 200}, {"x": 95, "y": 145}, {"x": 131, "y": 85}, {"x": 120, "y": 65},
  {"x": 100, "y": 80}, {"x": 69, "y": 140}, {"x": 62, "y": 200}, {"x": 76, "y": 260}, {"x": 120, "y": 320}, {"x": 170, "y": 350}, {"x": 270, "y": 365}, {"x": 600, "y": 365}, {"x": 695, "y": 334}, {"x": 745, "y": 280}, {"x": 769, "y": 210}, {"x": 763, "y": 140}, {"x": 721, "y": 70}, {"x": 650, "y": 25}, {"x": 550, "y": 18}, {"x": 220, "y": 18}, {"x": 110, "y": 70}, {"x": 131, "y": 85}
];
  const membrane = cellAreaGEnter
    .append('path')
      .attr('id', 'plasma membrane')
      .attr('class', 'component')
      .attr('d', lineFunction(membraneData));

//cytosol
const cytosolData = [
  {"x": innerWidth*0.33, "y": innerHeight*0.25}, {"x": innerWidth*0.33, "y": innerHeight*0.2}, {"x": innerWidth*0.35, "y": innerHeight*0.16}, {"x": innerWidth*0.4, "y": innerHeight*0.14}, {"x": innerWidth*0.43, "y": innerHeight*0.18}, {"x": innerWidth*0.43, "y": innerHeight*0.32}, {"x": innerWidth*0.45, "y": innerHeight*0.355}, {"x": innerWidth*0.445, "y": innerHeight*0.4}, {"x": innerWidth*0.435, "y": innerHeight*0.55}, {"x": innerWidth*0.46, "y": innerHeight*0.66}, {"x": innerWidth*0.4, "y": innerHeight*0.69}, {"x": innerWidth*0.39, "y": innerHeight*0.55}, {"x": innerWidth*0.3, "y": innerHeight*0.54},
  {"x": innerWidth*0.29, "y": innerHeight*0.6}, {"x": innerWidth*0.29, "y": innerHeight*0.70}, {"x": innerWidth*0.24, "y": innerHeight*0.75}, {"x": innerWidth*0.2, "y": innerHeight*0.8}, {"x": innerWidth*0.14, "y": innerHeight*0.7}, {"x": innerWidth*0.115, "y": innerHeight*0.45}, {"x": innerWidth*0.15, "y": innerHeight*0.3}, {"x": innerWidth*0.26, "y": innerHeight*0.31}, {"x": innerWidth*0.265, "y": innerHeight*0.43}, {"x": innerWidth*0.22, "y": innerHeight*0.39}, {"x": innerWidth*0.18, "y": innerHeight*0.4}, {"x": innerWidth*0.16, "y": innerHeight*0.53}, {"x": innerWidth*0.18, "y": innerHeight*0.67},
  {"x": innerWidth*0.23, "y": innerHeight*0.67}, {"x": innerWidth*0.25, "y": innerHeight*0.65}, {"x": innerWidth*0.24, "y": innerHeight*0.55}, {"x": innerWidth*0.24, "y": innerHeight*0.45}, {"x": innerWidth*0.28, "y": innerHeight*0.46}, {"x": innerWidth*0.34, "y": innerHeight*0.46}, {"x": innerWidth*0.36, "y": innerHeight*0.35}, {"x": innerWidth*0.33, "y": innerHeight*0.25}
];
  const cytosol = cellAreaGEnter
    .append('path')
      .attr('id', 'cytosol')
      .attr('class', 'component')
      .attr('d', lineFunction(cytosolData));

//cytoskeleton
const cytoskeletonData = [{"x": 280, "y": 279}, {"x": 480, "y": 285}, {"x": 470, "y": 328}, {"x": 220, "y": 328}, {"x": 220, "y": 280}, {"x": 340, "y": 280}];
  const cytoskeleton = cellAreaGEnter
    .append('path')
      .attr('id', 'cytoskeleton')
      .attr('class', 'component')
      .attr('d', lineFunction(cytoskeletonData));

//endoplasmic reticulum
const erData = [{"x": 540, "y": 200}, {"x": 535, "y": 160}, {"x": 530, "y": 120}, {"x": 538, "y": 100}, {"x": 550, "y": 103}, {"x": 580, "y": 110}, {"x": 590, "y": 130}, {"x": 597, "y": 230}, {"x": 550, "y": 295}, {"x": 510, "y": 270}, {"x": 530, "y": 230}, {"x": 540, "y": 200}];
  const endoplasmicReticulum = cellAreaGEnter
    .append('path')
      .attr('id', 'endoplasmic reticulum')
      .attr('class', 'component')
      .attr('d', lineFunction(erData));

//nucleus
  const nucleus = cellAreaGEnter
    .append('path')
      .attr('id', 'nucleus')
      .attr('class', 'component')
      .attr('transform', `translate(${innerWidth*0.54}, ${innerHeight * 0.5})`)
      .attr('d', arc()({
        innerRadius: 0,
        outerRadius: 80,
        startAngle: Math.PI,
        endAngle: Math.PI*4
      }))
    .merge(cellAreaG);

//ribosome
const ribosomeData = [{"x": 590, "y": 107}, {"x": 550, "y": 70}, {"x": 570, "y": 50}, {"x": 655, "y": 65}, {"x": 660, "y": 130}, {"x": 615, "y": 152}, {"x": 590, "y": 107}];
  const ribosome = cellAreaGEnter
    .append('path')
      .attr('id', 'ribosome')
      .attr('class', 'component')
      .attr('d', lineFunction(ribosomeData));

//mitochondrion
const mitochondrionData = [{"x": 140, "y": 190}, {"x": 160, "y": 155}, {"x": 195, "y": 160}, {"x": 195, "y": 210}, {"x": 205, "y": 235}, {"x": 195, "y": 250}, {"x": 170, "y": 255}, {"x": 145, "y": 237}, {"x": 140, "y": 190} ];
  const mitochondrion = cellAreaGEnter
    .append('path')
      .attr('id', 'mitochondrion')
      .attr('class', 'component')
      .attr('d', lineFunction(mitochondrionData));

//endosome
const endosomeData = [{"x": 135, "y": 88}, {"x": 160, "y": 70}, {"x": 190, "y": 56}, {"x": 250, "y": 46}, {"x": 280, "y": 50}, {"x": 250, "y": 100}, {"x": 200, "y": 110}, {"x": 150, "y": 100}, {"x": 135, "y": 88}];
  const endosome = cellAreaGEnter
    .append('path')
      .attr('id', 'endosome')
      .attr('class', 'component')
      .attr('d', lineFunction(endosomeData));

//lysosome
  const lysosome = cellAreaGEnter
    .append('path')
      .attr('id', 'lysosome')
      .attr('class', 'component')
      .attr('transform', `translate(${innerWidth*0.313}, ${innerHeight * 0.365})`)
      .attr('d', arc()({
        innerRadius: 0,
        outerRadius: 35,
        startAngle: Math.PI,
        endAngle: Math.PI*4
      }));

//peroxisome
  const peroxisome = cellAreaGEnter
    .append('path')
      .attr('id', 'peroxisome')
      .attr('class', 'component')
      .attr('transform', `translate(${innerWidth*0.341}, ${innerHeight * 0.635})`)
      .attr('d', arc()({
        innerRadius: 0,
        outerRadius: 35,
        startAngle: Math.PI,
        endAngle: Math.PI*4
      }));

//golgi apparatus
const golgiData = [{"x": 602, "y": 190}, {"x": 599, "y": 165}, {"x": 620, "y": 175}, {"x": 620, "y": 155}, {"x": 650, "y": 165}, {"x": 670, "y": 185}, {"x": 670, "y": 255}, {"x": 650, "y": 265}, {"x": 600, "y": 260}, {"x": 595, "y": 240}, {"x": 602, "y": 190}];
  const golgi = cellAreaGEnter
    .append('path')
      .attr('id', 'golgi')
      .attr('class', 'component')
      .attr('d', lineFunction(golgiData));

//centrosome
  const centrosome = cellAreaGEnter
    .append('path')
      .attr('id', 'centrosome')
      .attr('class', 'component')
      .attr('transform', `translate(${innerWidth*0.464}, ${innerHeight * 0.265})`)
      .attr('d', arc()({
        innerRadius: 0,
        outerRadius: 26,
        startAngle: Math.PI,
        endAngle: Math.PI*4
      }));

// Control the attributes (Coloring)
  // Apply coloring to each component depending on net_DEG value of locationData
  const controller = selectAll('.component').data(locationData);
  controller
      .attr('opacity', d =>
        d.location === selectedLocation
          ? 0.95
          : 0.55
      )
      .attr('fill', d => 
        +d.net_DEG > 0
          ? redScale(+d.net_DEG)
          : ( +d.net_DEG === 0 ? 'white' : blueScale(+d.net_DEG) )
      )
      .on('click', d => onClick(
        d.location === selectedLocation
          ? null
          : d.location
      ));

// Show tooltip on mouseover
  controller.append('title')
    .text(d => d.location + ': ' + d.net_DEG);
};